"""
by www.ixi-software.net
license :GPL

check readme.txt file 
"""


import wx
import osc
import time


class OSCWidget(object):
    """ abstract widget base class that sends via OSC its value when interacted. 
    """
    def __init__(self, oscaddress):
        self.oscaddress = oscaddress
        self.intEventList = [ wx.EVT_TOGGLEBUTTON.evtType[0], wx.EVT_BUTTON.evtType[0],
                              wx.EVT_COMMAND_SCROLL.evtType[0],  wx.EVT_SPIN.evtType[0]
                              ]
##        self.start()
##
##    def start(self): pass # to extend by user

    def action(self, e):
        if e.GetEventType() in self.intEventList :
            data = e.GetInt()
        else :
            data = str(e.GetString()) # 
            
##        print data, e.GetInt(), e.GetString()
        osc.sendMsg(self.oscaddress, [data], Server.ip, Server.outPort)
        

class Label(wx.StaticText) :
    def __init__(self, label="toogle", pos=wx.DefaultPosition, size=wx.DefaultSize, bgcol='Yellow'):
        wx.StaticText.__init__(self, Server.frame, -1, label, pos, size)
##        self.SetBackgroundColour(bgcol)
##        font = wx.Font(18, wx.Swiss, wx.Normal, wx.Normal)
##        self.SetText(font)

class Toggle(wx.ToggleButton, OSCWidget):
    def __init__(self, oscaddress, label="X", pos=wx.DefaultPosition, size=wx.DefaultSize):
        wx.ToggleButton.__init__(self, Server.frame, -1, label, pos, size)
        OSCWidget.__init__(self, oscaddress)
        Server.frame.Bind(wx.EVT_TOGGLEBUTTON, self.action, self)


class Button(wx.Button, OSCWidget):
    def __init__(self, oscaddress, label="O", pos=wx.DefaultPosition, size=wx.DefaultSize):
        wx.Button.__init__(self, Server.frame, -1, label, pos, size)
        OSCWidget.__init__(self, oscaddress)
        Server.frame.Bind(wx.EVT_BUTTON, self.action, self)


class Slider(wx.Slider, OSCWidget):
    def __init__(self, oscaddress, label="", pos=wx.DefaultPosition,
                 size=wx.DefaultSize, range=(0, 127), style='horizontal'):
        if style == 'vertical' or style == 'v':
            style = wx.SL_VERTICAL | wx.SL_LABELS
            if size == wx.DefaultSize : size = (-1, 127)
        else:
            style = wx.SL_HORIZONTAL | wx.SL_LABELS
            if size == wx.DefaultSize : size = (127, -1)
                
        wx.Slider.__init__(self, Server.frame, -1, value=25, minValue=range[0], maxValue=range[1],
                           pos=pos, size=size, style=style )
        OSCWidget.__init__(self, oscaddress)
        Server.frame.Bind(wx.EVT_COMMAND_SCROLL_THUMBTRACK, self.action, self)


class Choice(wx.Choice, OSCWidget):
    def __init__(self, oscaddress, label="", pos=wx.DefaultPosition, size=wx.DefaultSize, choices = ['zero', 'one', 'two'] ):
        wx.Choice.__init__(self, Server.frame, -1, pos=pos, size=size, choices = choices)
        OSCWidget.__init__(self, oscaddress)
        Server.frame.Bind(wx.EVT_CHOICE, self.action, self)


class RadioBox(wx.RadioBox, OSCWidget):
    def __init__(self, oscaddress, label="", pos=wx.DefaultPosition, size=wx.DefaultSize,
                 choices = ['zero', 'one', 'two'] , selected=0):
        wx.RadioBox.__init__(self, Server.frame, -1, label, pos=pos, size=size, choices = choices ,
                             majorDimension = 1, style=wx.RA_SPECIFY_COLS )
        OSCWidget.__init__(self, oscaddress)
        Server.frame.Bind(wx.EVT_RADIOBOX, self.action, self)
        self.SetSelection(selected) # which one is selected by default
        

class Number(wx.TextCtrl, OSCWidget):
    def __init__(self, oscaddress, label="", pos=wx.DefaultPosition, size=(30, 25),
                 value=0, range=(0, 127) ):
        wx.TextCtrl.__init__(self, Server.frame, -1, str(value), pos=pos, size=size)
        OSCWidget.__init__(self, oscaddress)

        spos = ( pos[0] + self.GetSize()[0],  pos[1] )
        ssize = ( 15, self.GetSize()[1] )
        spin = wx.SpinButton(Server.frame, -1,  spos, ssize, wx.SP_VERTICAL)
        spin.SetRange(range[0], range[1])
        spin.SetValue(value)

        Server.frame.Bind(wx.EVT_SPIN, self.action, spin)
        
    def action(self, e) : # extends superclass action
        OSCWidget.action(self, e)
        self.SetValue(str(e.GetPosition()))


## OSC zombi ################################
def wLabel(*msg) : Label(msg[0][2])
def wToggle(*msg) : Toggle(msg[0][2])
def wButton(*msg) : Button(msg[0][2])
def wSlider(*msg) : Slider(msg[0][2])
def wChoice(*msg) : Choice(msg[0][2])
def wRadioBox(*msg) : RadioBox(msg[0][2])
def wNumber(*msg) : Number(msg[0][2])

def bindZombi():
    osc.bind(wLabel, "/label")
    osc.bind(wToggle, "/toggle")
    osc.bind(wButton, "/button")
    osc.bind(wSlider, "/slider")
    osc.bind(wChoice, "/choice")
    osc.bind(wRadioBox, "/radiobox")
    osc.bind(wNumber, "/number")
###################################################################

class FpsTimer(wx.Timer):
    def __init__(self, fps, f):
        wx.Timer.__init__(self)
        self.Start(1000./fps)
        self.f = f # function to call on notify

    def Notify(self): self.f()






class Server(wx.App):
    """ opens a wx pannel, inits OSC and waits for welcome message from client.
    doFrame defines the widgets in the panel. They can be created via OSC zombi commands in this module
    """
    outPort = 0
    inPort = 0
    ip = 0
    frame = 0

    def __init__(self, name='app', pos=wx.DefaultPosition, size=wx.DefaultSize, fps=15, ip='127.0.0.1',
                 outPort=9001, inPort=9000, app='chuck', patch='oscServer.ck'):
        wx.App.__init__(self, 0)

        self.connected = 0
        self.fps = fps
        self.zombi = 0 # am i controlled by someone else via osc?
        
        # OSC
        Server.ip = ip
        Server.outPort = outPort
        Server.inPort = inPort
        osc.init()
        self.oscIn = osc.createListener(port=inPort) # in this case just using one socket
        osc.bind(self.status, "/connected") #
        osc.bind(self.setSize, "/size") #
        bindZombi() # all zombi commands
        osc.sendMsg('/connected', [1], port=Server.outPort) # all ready, send notification

        self.t = FpsTimer(self.fps, self.step) # for the incomming osc loop
        
        # contructing the frame 
        Server.frame = wx.Frame(None, -1, name, pos, size)
        Server.frame.CreateStatusBar(1,0)
        Server.frame.SetStatusText("OSC status : waiting for OSC connection")

        # wait for a sucessful connected message from sound server
        past = time.time()
        while not self.connected or not self.zombi : # wait 5 secs for welcome message
            time.sleep(0.1)
            osc.getOSC(self.oscIn)
            if time.time() - past > 5 :
                if not self.connected:
                    Server.frame.SetStatusText('I didnt get a connected message from sound server')
                break

        self.doWidgets() # users place their code here. define the widgets

        Server.frame.Show(True)
        self.SetTopWindow(Server.frame)

        print 'wxwidgets server up and runming'
        self.MainLoop() # main loop
        
        osc.sendMsg('/quit', [1], port=Server.outPort) # app has exited the main loop, quiting

    def doFrame(self) : pass
    def step(self): osc.getOSC(self.oscIn) # check for incoming msgs
    def setSize(*msg): self.size = msg[0][2]

    def status(self, *msg):
        print "connected msg received from sound server"
        self.connected = msg[0][2]
        if self.connected :
            s = "OSC status : ok. Receiving %i . Sending %s, %i" % (Server.inPort, Server.ip, Server.outPort)
            Server.frame.SetStatusText(s)

    def launchPatch(self, app, patch) :
        cmd = app+' '+patch
        pid = wx.Execute(cmd, wx.EXEC_ASYNC) #, self.process)



if __name__ == '__main__':
    Server(size=(800, 600), fps=10)
